<?php

/**
 * @author: Andrii Kopyniak
 * @ctime: 05.04.2012
 * @mtime: 23.11.2012
 *
 * @description: iPhone Controlled VOD v.3.0
 */

error_reporting (E_ALL);

require_once "do_config.php";
require_once "utils.php";

require_once "config_parsers.php";
require_once "m3u.class.php";
require_once "smbtree.class.php";
require_once "sources.class.php";
require_once "favorites.class.php";
require_once "network_folder.class.php";


#************************************************#
# Short manual
#************************************************#
# cmd = ls ([folder_url]) | pls ([ctr, url, name])
# | favorite ([ctr, name]) | network
# ctr = get (url) | put (url) | append (url) 
# | move (url, name) | make (name)
# | remove (url | name)
#
# do := [? [cmd,ctr,folder_url,url,name]
#************************************************#

////////////////////////////////////////////////////////////////////////////////
// Setting
////////////////////////////////////////////////////////////////////////////////
$time_start = microtime (true);
header ('Content-type: text/xml; charset=utf-8', true);

////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Input
////////////////////////////////////////////////////////////////////////////////
// main command
$CMD  = safely_get_value_of_global_variables ($_GET, 'cmd');
// list directory
$FOLDER_URL = safely_get_value_of_global_variables ($_GET, 'folder_url');
$HEAD = safely_get_value_of_global_variables ($_GET, 'head');
$TAIL = safely_get_value_of_global_variables ($_GET, 'tail');

// playlist manager
$CTR  = safely_get_value_of_global_variables ($_GET, 'ctr');
$URL  = urldecode (safely_get_value_of_global_variables ($_GET, 'url'));
$NAME = basename (safely_get_value_of_global_variables ($_GET, 'name'));

// favorite manager
$MEDIA_URL = safely_get_value_of_global_variables ($_GET, 'media_url');

$DEBUG_LEVEL  = safely_get_value_of_global_variables ($_GET, 'debug_level');
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Begin
////////////////////////////////////////////////////////////////////////////////

$xml_template = "<?xml version='1.0' encoding='UTF-8' ?>
  <command_result>
    <param name='protocol_version' value='". do_config::PROTOCOL_VERSION ."'/>
  </command_result>";

$xml = simplexml_load_string ($xml_template);

if ($DEBUG_LEVEL != 0)
{
  add_item ($xml, 'debug', 'plugin_name', DuneSystem::$properties['plugin_name']);
  add_item ($xml, 'debug', 'install_dir_path', DuneSystem::$properties['install_dir_path']);
  add_item ($xml, 'debug', 'tmp_dir_path', DuneSystem::$properties['tmp_dir_path']);
  add_item ($xml, 'debug', 'plugin_www_url', DuneSystem::$properties['plugin_www_url']);
  add_item ($xml, 'debug', 'plugin_cgi_url', DuneSystem::$properties['plugin_cgi_url']);
  add_item ($xml, 'debug', 'data_dir_path', DuneSystem::$properties['data_dir_path']);
}

switch (strtolower ($CMD)) // cmd = ls | pls "
{
  case 'pls':
  {
    switch (strtolower ($CTR))
    {
      case 'get':
      {
        if (is_dir (do_config::$PLAYLIST_DIR))
        {
          if (file_exists ($URL) && filetype ($URL) == "file")
          {
            $ext = explode_ext ('.', $URL, -2);
  
            switch (strtolower (isset ($ext[1]) ? ($ext[1]) : ('')) )
            {
            case 'm3u': case 'm3u8':
            {
              $m3uParser = new m3uParser ($URL);
  
              if (count ($m3uParser->getLocations ()))
              {
  
                $command_status = $xml->addChild ('param');
                $command_status->addAttribute ('name', 'command_status');
                $command_status->addAttribute ('value', 'ok');
  
                $player_state = $xml->addChild ('param');
                $player_state->addAttribute ('name', 'player_state');
                $player_state->addAttribute ('value', (string)$CMD);
  
                $pls_ctr = $xml->addChild ('param');
                $pls_ctr->addAttribute ('name', 'pls_ctr');
                $pls_ctr->addAttribute ('value', (string)$CTR);
  
                $pls_name = $xml->addChild ('param');
                $pls_name->addAttribute ('name', 'pls_name');
                $pls_name->addAttribute ('value', (string)$URL);
  
                $pls_ver = $xml->addChild ('param');
                $pls_ver->addAttribute ('name', 'pls_ver');
                $pls_ver_val = $m3uParser->getVersion ();
                $pls_ver->addAttribute ('value', ($pls_ver_val <= 1) ? '#M3U' : '#EXTM3U');
  
                $items = $xml->addChild ('items');
  
                foreach($m3uParser->getLocations () as $key => $value)
                {
                  $item = $items->addChild ('item');
  
                  if ($pls_ver_val == 2)
                  {
                    $m3uParser->getExtInfo ($key, $len, $title);
  
                    add_item ($item, 'extinf', $len, $title);
                  }
  
                  add_item ($item, 'param', 'url', (string)$value);
                }
              }
              break;
            }
            default:
              make_error ($xml, "invalid_parameters", "'url' parameter is missing");
              break;
            }
          }
          else
          {
            make_error ($xml, "invalid_parameters", "'url' parameter is missing");
          }
        }
        else
        {
          make_error ($xml, "internal_error", __LINE__ . ": Permission denied or environment fail");
        }
        break;
      }
      case 'put': case 'append':
      {
        if (is_dir (do_config::$PLAYLIST_DIR))
        {
          if (file_exists($URL) && filetype($URL) == "file")
          {
            $ext = explode_ext ('.', $URL, -2);
  
            switch ( strtolower(isset($ext[1])?($ext[1]):('')) )
            {
            case 'm3u': case 'm3u8':
  
              $GET_HTTP_RAW_POST_DATA = "<?xml version='1.0' ?><empty></empty>";
              $HTTP_RAW_POST_DATA = ((isset($HTTP_RAW_POST_DATA))?($HTTP_RAW_POST_DATA):($GET_HTTP_RAW_POST_DATA));
              $input_xml = simplexml_load_string((isset($HTTP_RAW_POST_DATA))?($HTTP_RAW_POST_DATA):($GET_HTTP_RAW_POST_DATA));
  
              if ($input_xml->xpath('/items') != false)
              {
                $command_status = $xml->addChild('param');
                $command_status->addAttribute('name', 'command_status');
                $command_status->addAttribute('value', 'ok');
  
                $player_state = $xml->addChild('param');
                $player_state->addAttribute('name', 'player_state');
                $player_state->addAttribute('value', (string)$CMD);
  
                $pls_ctr = $xml->addChild('param');
                $pls_ctr->addAttribute('name', 'pls_ctr');
                $pls_ctr->addAttribute('value', (string)$CTR);
  
                $playlist_data = '';
                if ($input_xml->xpath('/items/item/param') != false)
                {
                  $pls_items = $xml->addChild('items');
  
                  foreach ($input_xml->item as $item)
                  {
                    foreach ($item->param as $param)
                    {
                      switch( strtolower((string) $param['name']))
                      { 
                        // Получение атрибутов элемента по индексу
                      case 'url':
                        $pls_item = $pls_items->addChild('item');
                        $pls_param = $pls_item->addChild('param');
                        $pls_param->addAttribute('key', 'url');
                        $pls_param->addAttribute('value', $param['value']);
                        $playlist_data .= $param['value']."\n";
                        break;
                      }
                    }
                  }
                }
  
                switch( strtolower($CTR))
                {
                case 'put':
                  $fp = fopen($URL, 'w');
                  break;
                case 'append':
                  $fp = fopen($URL, 'a');
                  break;
                }
                if ($fp)
                {
                  fwrite($fp, $playlist_data);
                  fclose($fp);
                }
                else
                {
                  make_error($xml, "internal_error", "Permission denied");
                }
              }
              else
              {
                make_error($xml, "operation_failed", "unsupported format data");
                $pls_raw_data = $xml->addChild('http_raw_post_data', $HTTP_RAW_POST_DATA);
              }
              break;
            default:
              make_error($xml, "invalid_parameters", "'url' parameter is missing");
              break;
            }
          }
          else
          {
            make_error($xml, "operation_failed", "URL doesn`t exist");
          }
        }
        else
        {
          make_error($xml, "internal_error", __LINE__ . ": Permission denied or environment fail");
        }
        break;
      }
      case 'move':
      {
        if (is_dir(do_config::$PLAYLIST_DIR))
        {
  
          if (file_exists($URL) && filetype($URL) == "file")
          {
            $ext = explode_ext ('.', $URL, -2);
  
            switch ( strtolower(isset($ext[1])?($ext[1]):('')) )
            {
            case 'm3u': case 'm3u8':
              $OLDNAME=$URL;
              $ONLY_PATH = explode_ext ('/', $URL, -2);
              $NEWNAME=$ONLY_PATH[0] . "/" . $NAME;
  
              if (rename($OLDNAME, $NEWNAME))
              {
                $command_status = $xml->addChild('param');
                $command_status->addAttribute('name', 'command_status');
                $command_status->addAttribute('value', 'ok');
  
                $player_state = $xml->addChild('param');
                $player_state->addAttribute('name', 'player_state');
                $player_state->addAttribute('value', (string)$CMD);
  
                $pls_ctr = $xml->addChild('param');
                $pls_ctr->addAttribute('name', 'pls_ctr');
                $pls_ctr->addAttribute('value', (string)$CTR);
              }
              else
              {
                make_error($xml, "internal_error", "Permission denied");
              }
              break;
            default:
              make_error($xml, "invalid_parameters", "'url' parameter is missing");
              break;
            }
          }
          else
          {
            make_error($xml, "invalid_parameters", "'url' parameter is missing");
          }
        }
        else
        {
          make_error($xml, "internal_error", __LINE__ . ": Permission denied or environment fail");
        }
        break;
      }
      case 'make':
      {
        if (is_dir(do_config::$PLAYLIST_DIR))
        {
          $ext = explode_ext ('.', do_config::$PLAYLIST_DIR . $NAME, -2);
  
          switch ( strtolower(isset($ext[1])?($ext[1]):('')) )
          {
          case 'm3u': case 'm3u8':
            if (!file_exists(do_config::$PLAYLIST_DIR . $NAME))
            {
              if (fopen(do_config::$PLAYLIST_DIR . $NAME, "x"))
              {
                $command_status = $xml->addChild('param');
                $command_status->addAttribute('name', 'command_status');
                $command_status->addAttribute('value', 'ok');
  
                $player_state = $xml->addChild('param');
                $player_state->addAttribute('name', 'player_state');
                $player_state->addAttribute('value', (string)$CMD);
  
                $pls_ctr = $xml->addChild('param');
                $pls_ctr->addAttribute('name', 'pls_ctr');
                $pls_ctr->addAttribute('value', (string)$CTR);
              }
              else
              {
                make_error($xml, "internal_error", "Permission denied");
              }
            }
            else
            {
              make_error($xml, "internal_error", "file '$NAME' is exists");
            }
            break;
          default:
            make_error($xml, "invalid_parameters", "'url' parameter is missing");
            break;
          }
        }
        else
        {
          if (!mkdir(do_config::$PLAYLIST_DIR))
            make_error($xml, "internal_error", "Permission denied");
        }
        break;
      }
      case 'remove':
      {
        if (is_dir(do_config::$PLAYLIST_DIR))
        {
          if (file_exists($URL) && filetype($URL) == "file")
          {
            $ext = explode_ext ('.', $URL, -2);
  
            switch ( strtolower(isset($ext[1])?($ext[1]):('')) )
            {
            case 'm3u': case 'm3u8':
              if (unlink($URL))
              {
                $command_status = $xml->addChild('param');
                $command_status->addAttribute('name', 'command_status');
                $command_status->addAttribute('value', 'ok');
  
                $player_state = $xml->addChild('param');
                $player_state->addAttribute('name', 'player_state');
                $player_state->addAttribute('value', (string)$CMD);
  
                $pls_ctr = $xml->addChild('param');
                $pls_ctr->addAttribute('name', 'pls_ctr');
                $pls_ctr->addAttribute('value', (string)$CTR);
              }
              else
              {
                make_error($xml, "internal_error", "Permission denied");
              }
              break;
            default:
              make_error($xml, "invalid_parameters", "'url' parameter is missing");
              break;
            }
          }
          else
          {
            make_error($xml, "invalid_parameters", "'url' parameter is missing");
          }
        }
        else
        {
          make_error($xml, "internal_error", __LINE__ . ": Permission denied or environment fail");
        }
        break;
      }
      default:
      {
        unset($CTR);
  
        if ( ! is_dir (do_config::$PLAYLIST_DIR))
        {
          if ( ! mkdir (do_config::$PLAYLIST_DIR))
            make_error($xml, "internal_error", "Permission denied");
        }
  
        if ( is_dir(do_config::$PLAYLIST_DIR))
        {
          $popd = getcwd ();
          chdir (do_config::$PLAYLIST_DIR);
          $devices = glob (do_config::$PLAYLIST_DIR, (GLOB_MARK | GLOB_ONLYDIR));
          chdir ($popd);
  
          if ($devices != false)
          {
            $FOLDER_URL = $devices[0];
          }
          else
          {
            $FOLDER_URL = do_config::$PLAYLIST_DIR;
            if ( !mkdir ($FOLDER_URL))
            {
              make_error ($xml, "internal_error", "Permission denied");
              break;
            }
          }
          do_config::$ROOT_DIR = '';
        }
        break;
      }
    }
    if ( !empty ($CTR))
      break;
  }

  case 'ls':
  {
    if ( is_dir (do_config::$ROOT_DIR . $FOLDER_URL) )
    {
      if (strcmp ($CMD,'ls') == 0)
      {
        $popd = getcwd ();
        chdir (do_config::$ROOT_DIR . $FOLDER_URL);
        $current_dir = getcwd ();
  
        if (!preg_match ('#^'. do_config::$ROOT_DIR .'#i', $current_dir))
          $FOLDER_URL = '';
  
        chdir ($popd);
      }
  
      $command_status = $xml->addChild ('param');
      $command_status->addAttribute ('name', 'command_status');
      $command_status->addAttribute ('value', 'ok');
  
      $player_state = $xml->addChild ('param');
      $player_state->addAttribute ('name', 'player_state');
      $player_state->addAttribute ('value', (string)$CMD);
  
      $items = $xml->addChild ('items');
  
      if (strlen ($HEAD))
      {
        $options = array
          (
            'head' => $HEAD,
            'tail' => $TAIL
          );
        $dirs[] = do_config::$ROOT_DIR . $FOLDER_URL;
        $files = list_directory_contents ($options, $dirs);
      } 
      else
      {
        $files = scandir (do_config::$ROOT_DIR . $FOLDER_URL);
      }
  
      foreach ($files as $key => $value)
      {
        $item = $items->addChild ('item');
  
        $ls_name = $item->addChild ('param');
        $ls_name->addAttribute ('name', 'ls_name');
        $ls_name->addAttribute ('value', (string)$value);
  
        if (!strcasecmp (filetype (do_config::$ROOT_DIR . $FOLDER_URL .'/'. (string)$value), 'dir'))
        {
          $ls_type = $item->addChild('param');
          $ls_type->addAttribute('name', 'ls_type');
          $ls_type->addAttribute('value', "folder");
  
          $ls_icon_url = $item->addChild('param');
          $ls_icon_url->addAttribute('name', 'ls_icon_url');
          $ls_icon_url->addAttribute('value', '');
  
          $stat = stat(do_config::$ROOT_DIR . $FOLDER_URL .'/'. $value);
  
          /* executing function for current folder  */
          $metadata = array('file' => 0, 'dir' => 0);
          if (($value == '.') || ($value == '..'))
          {
            $ls_metadata = $item->addChild('param');
            $ls_metadata->addAttribute('name', 'ls_metadata');
            $ls_metadata->addAttribute('value', '');
          }
          else
          {
            $ls_metadata = $item->addChild('param');
            $ls_metadata->addAttribute('name', 'ls_metadata');
            $ls_metadata->addAttribute('value', '');
          }
  
          $ls_metadata->addAttribute('mime',
            mime_content_type (do_config::$ROOT_DIR . $FOLDER_URL .'/'. (string)$value));
        }
        else
        {
          $ls_type = $item->addChild('param');
          $ls_type->addAttribute('name', 'ls_type');

          switch (media_info::get_media_type (do_config::$ROOT_DIR . $FOLDER_URL .'/'. (string)$value, $item, $DEBUG_LEVEL))
          {
            case 'audio':
            {
              $ls_type->addAttribute('value', 'audio' );
              break;
            }
            case 'video':
            {
              $ls_type->addAttribute('value', 'video' );
              break;
            }
            case 'playlist':
            {
              $ls_type->addAttribute('value', 'playlist' );
              break;
            }
            case 'image':
            {
              $ls_type->addAttribute('value', 'image' );
              break;
            }
            default:
            {
              $ls_type->addAttribute('value', 'unknown' );
              break;
            }
          }

          $ls_type = $item->addChild('param');
          $ls_type->addAttribute('name', 'ls_pls_url');
          $ls_type->addAttribute('value', $FOLDER_URL . $value );
  
          $ls_icon_url = $item->addChild('param');
          $ls_icon_url->addAttribute('name', 'ls_icon_url');
          $ls_icon_url->addAttribute('value', '');
  
          $stat = stat(do_config::$ROOT_DIR . $FOLDER_URL . '/' . $value);
  
          $ls_metadata = $item->addChild('param');
          $ls_metadata->addAttribute('name', 'ls_metadata');
          $ls_metadata->addAttribute('value',
            'size: ' . format_bytes ($stat['size']));
  
          $ls_metadata->addAttribute('mime',
            mime_content_type (do_config::$ROOT_DIR . $FOLDER_URL . '/' . (string)$value));
        }
      }
    }
    else
    {
      make_error($xml, "invalid_parameters", __LINE__ . ": 'folder_url' parameter is missing - ".
        do_config::$ROOT_DIR . $FOLDER_URL);
      $FOLDER_URL='';
    }
  
    break;
  }
  
  // * protocol version 3
  case 'sources':
  {
    $src_dev = new sources ();
    $data = $src_dev->get_sources ();
  
    make_ok ($xml, $CMD);
  
    $items  = $xml->addChild ('items');
  
    foreach ($data as $name => $value)
    {
      $item = $items->addChild('item');
  
      add_item ($item, 'param', 'name', $value['name']);
      add_item ($item, 'param', 'type', $value['type']);
    }
  
    break;
  }
  
  case 'network_browser':
  {
    make_ok ($xml, $CMD);
  
    $items = $xml->addChild ('items');
    $item = $items->addChild('item');
  
    add_item ($item, 'param', 'name', 'Network Places');
    add_item ($item, 'param', 'type', 'smb');
  
    break;
  }
  
  case 'smb':
  {
    $smb_shares = new smbtree ();
  
    switch (safely_get_value_of_global_variables ($_GET, 'get'))
    {
    case 'xdomains':
    {
      make_ok ($xml, $CMD);
  
      $data = $smb_shares->get_xdomains ();
  
      $items  = $xml->addChild ('items');
  
      foreach ($data as $domain)
      {
        $item = $items->addChild('item');
        add_item ($item, 'param', 'name', $domain['name']);
        add_item ($item, 'param', 'comment', $domain['comment']);
      }
  
      break;
    }
  
    case 'domains':
    {
      make_ok ($xml, $CMD);
  
      $data = $smb_shares->get_domains ();
  
      $items  = $xml->addChild ('items');
  
      foreach ($data as $domain)
      {
        $item = $items->addChild('item');
        add_item ($item, 'param', 'name', $domain['name']);
        add_item ($item, 'param', 'comment', $domain['comment']);
      }
  
      break;
    }
  
    case 'workgroup-servers':
    {
      make_ok ($xml, $CMD);
  
      $data = $smb_shares->get_workgroup_servers ($NAME);
  
      $items  = $xml->addChild ('items');
  
      foreach ($data as $server)
      {
        $item = $items->addChild('item');
        add_item ($item, 'param', 'name', $server['name']);
        add_item ($item, 'param', 'comment', $server['comment']);
      }
      break;
    }
    case 'server-shares':
    {
      make_ok ($xml, $CMD);
  
      $data = $smb_shares->get_server_shares ($NAME);
  
      $items  = $xml->addChild ('items');
  
      foreach ($data as $share)
      {
        $item = $items->addChild('item');
        add_item ($item, 'param', 'name', $share['name']);
        add_item ($item, 'param', 'comment', $share['comment']);
      }
      break;
    }
    default:
      make_error($xml, "invalid_parameters", "'get' parameter is missing");
    }
  
    break;
  }
  
  case 'network_folder':
  {
    $network_folder_properties = new network_folder();
  
    if ($network_folder_properties->open ('/config/network_folders.properties'))
    {
      $MENU = safely_get_value_of_global_variables ($_GET, 'menu');
  
      $TYPE       = safely_get_value_of_global_variables ($_GET, 'type');
      $SERVER     = safely_get_value_of_global_variables ($_GET, 'server');
      $DIRECTORY  = safely_get_value_of_global_variables ($_GET, 'directory');
      $USER       = safely_get_value_of_global_variables ($_GET, 'user');
      $PASSWORD   = safely_get_value_of_global_variables ($_GET, 'password');
      $PROTOCOL   = safely_get_value_of_global_variables ($_GET, 'protocol');
  
      switch ($MENU)
      {
      case 'new':
      {
        if ($network_folder_properties->new_network_folder ($NAME, $TYPE, $SERVER, $DIRECTORY, $USER, $PASSWORD, $PROTOCOL))
        {
          $other_params = array
            (
              'menu' => $MENU,
              'name' => $NAME
  
            );
  
          make_ok ($xml, $CMD, $other_params);
        }
        else
          make_error ($xml, "internal_error", "Permission denied");
  
        break;
      }
      case 'edit':
      {
        if ($network_folder_properties->edit_network_folder ($NAME, $TYPE, $SERVER, $DIRECTORY, $USER, $PASSWORD, $PROTOCOL))
        {
          $other_params = array
            (
              'menu' => $MENU,
              'name' => $NAME
  
            );
  
          make_ok ($xml, $CMD, $other_params);
        }
        else
          make_error ($xml, "internal_error", "Permission denied");
  
        break;
      }
      case 'delete':
      {
        if ($network_folder_properties->delete_network_folder ($NAME))
        {
          $other_params = array
            (
              'menu' => $MENU,
              'name' => $NAME
  
            );
  
          make_ok ($xml, $CMD, $other_params);
        }
        else
          make_error ($xml, "internal_error", "Permission denied");
  
        break;
      }
  
      default:
      {
        if (!strcasecmp ('info', $MENU)) //case 'info': { break; }
        {
          $data = $network_folder_properties->get_info_from_network_folder ($NAME);
        }
        else
          $data = $network_folder_properties->get_properties ();
  
        make_ok ($xml, $CMD);
  
        $items  = $xml->addChild ('items');
  
        foreach ($data as $value)
        {
          $item   = $items->addChild('item');
  
          add_item ($item, 'param', 'name', $value['name']);
          add_item ($item, 'param', 'type', $value['type']);
          add_item ($item, 'param', 'server', $value['server']);
          add_item ($item, 'param', 'directory', $value['directory']);
  
          if (!strcasecmp ($value['type'], "0")) // SMB
          {
            add_item ($item, 'param', 'user', $value['user']);
            add_item ($item, 'param', 'password', $value['password']);
          }
          else //if (!strcasecmp ($value['type'], '1')) // NFS
          {
            add_item ($item, 'param', 'protocol', $value['protocol']);
          }
        }
        break;
      }
      }
  
      $network_folder_properties->close ();
    }
    else
    {
      make_error($xml, "internal_error", "Permission denied");
    }
  
    break;
  }
  
  case 'favorites':
  {
    $favorites_properties = new favorite_folder ();
  
    $MENU = safely_get_value_of_global_variables ($_GET, 'menu');
    $MEDIA_ACTION = safely_get_value_of_global_variables ($_GET, 'media_action');
    switch ($MENU)
    {
    case 'add':
    {
      if ($favorites_properties->add_item ($NAME, $MEDIA_URL))
      {
        $other_params = array
          (
            'menu' => $MENU,
            'name' => $NAME,
            'media_url' => $MEDIA_URL
          );
  
        if (strlen ($MEDIA_ACTION))
          $other_params['media_action'] = $MEDIA_ACTION;
  
        make_ok ($xml, $CMD, $other_params);
      }
      else
        make_error($xml, "internal_error", "Permission denied");
  
      break;
    }
    case 'remove':
    {
      if ($favorites_properties->remove_item ($NAME))
      {
        $other_params = array
          (
            'menu' => $MENU,
            'name' => $NAME
          );
  
        make_ok ($xml, $CMD, $other_params);
      }
      else
        make_error($xml, "internal_error", "Permission denied");
  
      break;
    }
    case 'power_on_action':
    {
      if ($favorites_properties->power_on_action ($NAME))
      {
        $other_params = array
          (
            'menu' => $MENU,
            'name' => $NAME
          );
  
        make_ok ($xml, $CMD, $other_params);
      }
      else
        make_error($xml, "internal_error", "Permission denied");
  
      break;
    }
    case 'power_off_action':
    {
      $favorites_properties->power_off_action ();
      $other_params = array
        (
          'menu' => $MENU,
        );
  
      make_ok ($xml, $CMD, $other_params);
  
      break;
    }
    default:
    {
      $power_on_action = $favorites_properties->get_power_on_action ();
  
      if ($favorites_properties->open_folder('/config/main_screen_items'))
      {
        $data = $favorites_properties->get_properties ();
  
        $other_params = array
          (
            'power_on_action' => $power_on_action
          );
        make_ok ($xml, $CMD, $other_params);
  
        $items  = $xml->addChild ('items');
  
        foreach ($data as $favorites_name => $favorites_value)
        {
          $item   = $items->addChild('item');
  
          add_item ($item, 'param', 'name', $favorites_name);
  
          $favorites_info_media_url = '';
          $favorites_info_media_action = '';
          foreach ($favorites_value as $favorites_info_key => $favorites_info_value)
          {
            if (!strcasecmp ($favorites_info_key, 'media_url'))
              $favorites_info_media_url = $favorites_info_value;
            else if (!strcasecmp ($favorites_info_key, 'media_action'))
              $favorites_info_media_action = $favorites_info_value;
  
            add_item ($item, 'param', $favorites_info_key, $favorites_info_value);
          }
  
          if (strlen ($favorites_info_media_url))
          {
            $favorites_media_url_type = media_info::get_media_type ($favorites_info_media_url, $item, $DEBUG_LEVEL);
            if (!strcasecmp ($favorites_media_url_type, 'unknown') && !strcasecmp ($favorites_info_media_action, 'browse'))
              add_item ($item, 'param', 'type', 'folder');
            else
              add_item ($item, 'param', 'type', $favorites_media_url_type);
          }
        }
  
        $favorites_properties->close_folder();
      }
      else
        make_error($xml, "internal_error", "Permission denied");
  
      break;
    }
    }
  
    break;
  }
  
  
  default:
  {
    make_error($xml, "unknown_command", "'cmd' parameter is missing");
    break;
  }
}

if ($DEBUG_LEVEL != 0)
  add_item ($xml, 'debug', 'end_time_in_seconds', (microtime (true) - $time_start));

echo $xml->asXML ();
////////////////////////////////////////////////////////////////////////////////
// END
////////////////////////////////////////////////////////////////////////////////
