<?php

/**
 * @author: Andrii Kopyniak
 * @date:   2013 Jan 10
 */

class smbtree
{
  private $cmd            = 'smbtree';
  private $descriptorspec = array ();
  private $cwd            = '/';
  private $env            = array ();
  private $smbtree_output = '';
  private $return_value   = 0;
  private $no_pass        = true;
  private $debuglevel     = 0;

  public function __construct ()
  {
    $this->cmd = '/tango/firmware/bin/smbtree ';
    $this->cwd = '/tmp';

    $this->descriptorspec = array
      (
        0 => array("pipe", "r"),
        1 => array("pipe", "w"),
        2 => array("pipe", "w")
      );
    $this->env = array ('LD_LIBRARY_PATH' => '/tango/firmware/lib');

    $no_pass = true;
  }
  
  private function get_auth_options ()
  {
    if ($this->is_no_pass ())
      return '-N';

    return '';
  }

  private function get_debug_level ()
  {
    return '--debuglevel ' . $this->debuglevel;
  }

  private function is_no_pass ()
  {
    return $this->no_pass;
  }

  /*
   * @return 0 if success
   */
  private function execute ($args = '')
  {
    $process = proc_open ($this->cmd
      . $this->get_auth_options () . ' '
      . $this->get_debug_level () . ' '
      . $args,
      $this->descriptorspec, $pipes, $this->cwd, $this->env);

    if (is_resource ($process))
    {
      //fclose ($pipes[0]);
      
      $this->smbtree_output = stream_get_contents ($pipes[1]);
      fclose ($pipes[1]);
      fclose ($pipes[2]);
    
      // Важно закрывать все каналы перед вызовом
      // proc_close во избежание мертвой блокировки
      $this->return_value = proc_close ($process);
    }

    return $this->return_value;
  }

  private function parse_smbtree_output ($input_lines)
  {
    $output = array ();

    if (!strlen ($input_lines))
      return array ();

    $output_lines = explode ("\n", $input_lines);
    if ($output_lines == false)
      return array ();

    foreach ($output_lines as $line)
    {
      if (strlen ($line))
      {
        $detail_info = explode ("\t", $line);

        if (count ($detail_info))
        {
          $output[] = array
            (
              'name'     => $detail_info[0],
              'comment'  => safely_get_value_of_global_variables ($detail_info, '1')
            );
        }
      }
    }

    return $output;
  }

  public function get_xdomains ()
  {
    if ($this->execute ('--xdomains') != 0)
      return array ();

    return $this->parse_smbtree_output ($this->smbtree_output);
  }

  public function get_domains ()
  {
    if ($this->execute ('--domains') != 0)
      return array ();

    return $this->parse_smbtree_output ($this->smbtree_output);
  }

  public function get_workgroup_servers ($domain)
  {
    if ($this->execute ('--workgroup-servers=' . $domain) != 0)
      return array ();

    return $this->parse_smbtree_output ($this->smbtree_output);
  }

  public function get_server_shares ($server)
  {
    if ($this->execute ('--server-shares=' . $server) != 0)
      return array ();

    return $this->parse_smbtree_output ($this->smbtree_output);
  }
}
