<?php

//file_put_contents ('/tmp/do.php.log', (string) time () . ': REQUEST_URI = '. $_ENV['REQUEST_URI'] . PHP_EOL, FILE_APPEND | LOCK_EX);

function make_error ($xml, $e_kind, $e_description)
{
  $command_status = $xml->addChild('param');
  $command_status->addAttribute('name', 'command_status');
  $command_status->addAttribute('value', 'failed');

  $error_kind = $xml->addChild('param');
  $error_kind->addAttribute('name', 'error_kind');
  $error_kind->addAttribute('value', (string)$e_kind);

  $error_description = $xml->addChild('param');
  $error_description->addAttribute('name', 'error_description');
  $error_description->addAttribute('value', (string)$e_description);

  $player_state = $xml->addChild('param');
  $player_state->addAttribute('name', 'player_state');
  $player_state->addAttribute('value', 'navigator');
}

function make_ok ($xml, $CMD, $OTHER_PARAMS = array ())
{
  $command_status = $xml->addChild ('param');
  $command_status->addAttribute ('name', 'command_status');
  $command_status->addAttribute ('value', 'ok');

  $player_state = $xml->addChild ('param');
  $player_state->addAttribute ('name', 'player_state');
  $player_state->addAttribute ('value', (string)$CMD);

  foreach ($OTHER_PARAMS as $key => $value)
  {
    $other_params = $xml->addChild('param');
    $other_params->addAttribute('name', (string) $key);
    $other_params->addAttribute('value', (string)$value);
  }
}

function add_item ($item, $item_name, $attr_name, $attr_value)
{
  $param = $item->addChild($item_name);
  $param->addAttribute('name', $attr_name);
  $param->addAttribute('value', $attr_value);
}

function get_metadata ($folder, &$metadata)
{
  $files = scandir($folder);
  foreach($files as $file)
  {
    if (($file == '.') || ($file == '..')) continue;
    $f0 = $folder.'/'.$file;
    if (is_dir($f0))
    {
      $metadata['dir'] = (int)$metadata['dir'] + 1;
    }
    else
    {
      $metadata['file'] = (int)$metadata['file']+1;
    }
  }
}

function explode_ext($delim, $str, $lim = 1)
{
  if ($lim > -2) return explode($delim, $str, abs($lim));

  $lim = -$lim;
  $out = explode($delim, $str);
  if ($lim >= count($out)) return $out;

  $out = array_chunk($out, count($out) - $lim + 1);

  return array_merge(array(implode($delim, $out[0])), $out[1]);
}

function format_bytes($a_bytes)
{
  if ($a_bytes < 1024)
  {
    return $a_bytes .' B';
  }
  elseif ($a_bytes < 1048576)
  {
    return round($a_bytes / 1024, 2) .' KB';
  }
  elseif ($a_bytes < 1073741824)
  {
    return round($a_bytes / 1048576, 2) . ' MB';
  }
  elseif ($a_bytes < 1099511627776)
  {
    return round($a_bytes / 1073741824, 2) . ' GB';
  }
  elseif ($a_bytes < 1125899906842624)
  {
    return round($a_bytes / 1099511627776, 2) .' TB';
  }
  elseif ($a_bytes < 1152921504606846976)
  {
    return round($a_bytes / 1125899906842624, 2) .' PB';
  }
  elseif ($a_bytes < 1180591620717411303424)
  {
    return round($a_bytes / 1152921504606846976, 2) .' EB';
  }
  elseif ($a_bytes < 1208925819614629174706176)
  {
    return round($a_bytes / 1180591620717411303424, 2) .' ZB';
  }
  else
  {
    return round($a_bytes / 1208925819614629174706176, 2) .' YB';
  }
}

function
list_directory_contents ($options, $dirs)
{
  $output = array ();

  $head = (int) safely_get_value_of_global_variables ($options, 'head');
  $tail = (int) safely_get_value_of_global_variables ($options, 'tail');

  if ($head < 0)
    $head = 0;

  if ($tail <= $head)
    $tail = $head + 1;

  foreach ($dirs as $dir)
  {
    if (is_dir ($dir))
    {
      $iterator = new DirectoryIterator ($dir);
      $iterator->seek ($head);
      while ($iterator->valid ())
      {
        if ($tail > $head++)
        {
          $output[] = $iterator->getFilename ();
          $iterator->next();
        }
        else
          break;
      }
    }
  }

  return $output;
}

class media_info
{
  public static function get_media_type ($path, $xml = null, $DEBUG_LEVEL = 0)
  {
    if ($DEBUG_LEVEL)
      add_item ($xml, 'debug_param', 'path', $path);

    if (preg_match ('#^[[:blank:]]*([^:]+)(://)(.*)$#', $path, $matches, PREG_OFFSET_CAPTURE))
      $scheme = safely_get_value_of_global_variables ($matches[1], '0');

    switch (strtolower ($scheme))
    {
      case 'plugin_launcher':
        return 'plugin';

      case '':
      {
        if (!file_exists ($path))
          return false;

        if (is_dir ($path))
        {
          if ($DEBUG_LEVEL)
            add_item ($xml, 'DEBUG', 'm_type', 'folder');
          return 'folder';
        }
        else if (is_file ($path))
        {
          return media_info::get_mime_content_type ($path, $xml, $DEBUG_LEVEL);
        }
      }
      case 'storage_name': // 'storage_label': 'storage_uuid':
      {
        $count = 1;
        $storage_path = str_replace ("storage_name", "storagename", $path, $count);
        $storage_path_components = parse_url ($storage_path);

        $all_fragment = strpbrk ($storage_path, '#');

        if (isset ($storage_path_components['fragment']))
          $storage_path_components['path'] = $storage_path_components['path'] .'#'. $storage_path_components['fragment'];
        else if (strlen ($all_fragment))
          $storage_path_components['path'] = $storage_path_components['path'] . $all_fragment;

        if (strlen (safely_get_value_of_global_variables ($storage_path_components, 'host'))
          && strlen (safely_get_value_of_global_variables ($storage_path_components, 'path')))
        {
          $check_storage_path = do_config::$ROOT_DIR . $storage_path_components['host'] . $storage_path_components['path'];
          if (file_exists ($check_storage_path))
          {
            if (is_dir ($check_storage_path))
            {
              if ($DEBUG_LEVEL)
                add_item ($xml, 'DEBUG', 'm_type', 'folder');
              return 'folder';
            }
            else if (is_file ($check_storage_path))
              return media_info::get_mime_content_type ($check_storage_path, $xml, $DEBUG_LEVEL);
          }
        }
      }
      default: // 'smb': 'nfs': 'nfs-tcp': 'nfs-udp': ...
        break;
    }

    return 'unknown';
  }
  
  public static function get_mime_content_type ($path, $xml = null, $DEBUG_LEVEL = 0)
  {
    $finfo = new finfo (FILEINFO_MIME_TYPE);
    $m_type = $finfo->file ($path); //$m_type = mime_content_type ($path);
    
    if ($DEBUG_LEVEL)
      add_item ($xml, 'debug_param', 'm_type', $m_type);

    if (preg_match ('/^(.+)\/.+$/', $m_type, $matches, PREG_OFFSET_CAPTURE))
    {
      $path_type = safely_get_value_of_global_variables ($matches[1], '0');
      if (strlen ($path_type))
      {
        $finfo->set_flags (FILEINFO_MIME_ENCODING);
        $m_encoding = $finfo->file ($path);
        if ($DEBUG_LEVEL)
          add_item ($xml, 'debug_param', 'm_encoding', $m_encoding);

        switch ($path_type)
        {
          case 'audio':
            return 'audio';
          case 'image':
            return 'image';
          case 'video':
            return 'video';
          case 'application':
            if (!strcasecmp ($m_encoding, 'binary'))
            {
              $finfo->set_flags (FILEINFO_NONE);
              $m_description = $finfo->file ($path);
              if ($DEBUG_LEVEL)
                add_item ($xml, 'debug_param', 'm_description', $m_description);

              switch (strtolower (pathinfo ($path, PATHINFO_EXTENSION)))
              {
                case 'mp3': case 'ac3': case 'wma': case 'ogg': case 'ogm': case 'm4a':
                case 'aif': case 'iff': case 'mid': case 'mpa': case 'ra': case 'wav':
                case 'flac': case 'ape': case 'vorbis': case 'aac': case 'a52':
                  return 'audio';

                case 'avi': case 'mp4': case 'mpg': case 'mpeg': case 'divx': case 'm4v':
                case '3gp': case 'asf': case 'wmv': case 'mkv': case 'mov': case 'ogv':
                case 'vob': case 'flv': case 'ts': case '3g2': case 'swf': case 'asf':
                case 'ps': case 'qt': case 'm2ts':
                  return 'video';

                case 'png': case 'jpg': case 'jpeg': case 'bmp': case 'gif': case 'psd':
                case 'pspimage': case 'thm': case 'tif': case 'yuf': case 'svg': case 'aai':
                case 'ico': case 'djpg': case 'dbmp': case 'dpng':
                  return 'image';

                case 'm3u': case 'm3u8': case 'pls':
                  return 'playlist';

                default:
                  return 'unknown';
              }
            }
          case 'text':
          {
            switch (pathinfo ($path, PATHINFO_EXTENSION))
            {
              /* TODO: exeptions list.  */
              case 'm3u': case 'm3u8': case 'pls':
                return 'playlist';
              default:
                return 'unknown';
            }
          }
          default:
            return 'unknown';
        }
      }
    }

    return 'unknown';
  }
}
